<?php

namespace App\Service;

use App\Exception;

class ProductService {

    private $util;
    private $ProductFactory;

    public function __construct (\App\Util\Util $util,
                                 \App\Factory\ProductFactory $ProductFactory) {
        $this->util = $util;
        $this->productFactory  = $ProductFactory;
    }
    
    public function getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda){
        
        $itens = $this->productFactory->getItensVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda);
        $arrayInfo = [];
        
        foreach($itens as $item){
                $itemInfo = [
                        'id'           => $item['NRSEQUITVEND'],
                        'name'         => $item['NMPRODUTO'],
                        'code'         => $item['CDARVPROD'],
                        'externalCode' => $item['CDPRODINTE'],
                        'quantity'     => $item['QTPRODVEND'],
                        'price'        => floatval($this->util->tofloat($item['VRUNITVEND'])),
                        'discount'     => floatval($this->util->tofloat($item['VRDESITVEND'])),
                        'addition'     => floatval($this->util->tofloat($item['VRACRITVEND'])),
                        'total'        => floatval($this->util->tofloat($item['TOTAL'])),
                        'fiscalInfo'   => [
                                            'ncm'          => $item['CDCLASFISC'],
                                            'cest'         => $item['CDCEST'],
                                            'cstIcms'      => $item['CDCSTPRODI'],
                                            'cstPisCofins' => $item['CDCSTPRODPC'],
                                            'cfop'         => $item['CDCFOPPROD'],
                                            'bcPis'        => floatval($this->util->tofloat($item['VRBCPISCOFINS'])),
                                            'vrAliqPis'    => floatval($this->util->tofloat($item['VRALIQPIS'])),
                                            'vrPis'        => floatval($this->util->tofloat($item['VRIMPPIS'])),
                                            'bcCofins'     => floatval($this->util->tofloat($item['VRBCPISCOFINS'])),
                                            'vrAliqCofins' => floatval($this->util->tofloat($item['VRALIQCOFINS'])),
                                            'vrCofins'     => floatval($this->util->tofloat($item['VRIMPCOFINS'])),
                                            'bcIcms'       => floatval($this->util->tofloat($item['VRBASECALCICMS'])),
                                            'vrAliqIcms'   => floatval($this->util->tofloat($item['VRPEALIMPFIS'])),
                                            'vrIcms'       => floatval($this->util->tofloat($item['VRIMPOPRODIT']))
                                        ]
                ];
            array_push($arrayInfo, $itemInfo);
        }
        
        return $arrayInfo;
    }
    
    public function getItens ($subsidiaryCode, $dateSale){
        $itensOrder = $this->productFactory->getItens($subsidiaryCode, $dateSale, $dateSale);
        if(empty($itensOrder))
            return;
            
        $arrayItens = [];
        $itensEst = [
                    'date'  => $dateSale,
                    'itens' => []
        ];
        
        foreach($itensOrder as $item){
        
            if($item['IDTIPOCOMPPROD'] == '3' ){
                $itensCombo = $this->productFactory->getItensCombo($item['CDFILIAL'], $item['CDCAIXA'] , $item['NRSEQVENDA'], $item['NRSEQUITVEND']);
                if(!empty($itensCombo))
                    array_push($arrayItens, $itensCombo);
            }else{
                
                $simpleProduct = [
                    'CDFILIAL'          => $item['CDFILIAL'],
                    'CDCAIXA'           => $item['CDCAIXA'],
                    'NRSEQVENDA'        => $item['NRSEQVENDA'],
                    'NRSEQUITVEND'      => $item['NRSEQUITVEND'],
                    'CDPRODUTO'         => $item['CDPRODUTO'],
                    'NMPRODUTO'         => $item['NMPRODUTO'],
                    'QTITVENDAEST'      => $item['QTPRODVEND'],
                    'IDUTILRETPRODACAB' => $item['IDUTILRETPRODACAB']
                ];
                array_push($arrayItens, $simpleProduct);
                
            }
        }

        foreach($arrayItens as $i){
            
            if(isset($i['IDUTILRETPRODACAB']) && $i['IDUTILRETPRODACAB'] == 'N'){
                $composicao = $this->productFactory->getComposicao($i['CDFILIAL'], $i['CDPRODUTO']);
                if(!empty($composicao)){
                   foreach ($composicao as $comp){
                        $itemComp = array (
                                'name'           => $comp['NMPRODUTO'],
                                'code'           => isset($comp['CDSUBPRODLOC']) ? $comp['CDSUBPRODLOC'] : $comp['CDSUBPRODUTO'],
                                'externalCode'   => '',
                                'unit'           => '',
                                'quantity'       => floatval($this->util->tofloat($i['QTITVENDAEST'])) * (isset($comp['QTSUBPRODLOC']) ? floatval($this->util->tofloat($comp['QTSUBPRODLOC'])) : floatval($this->util->tofloat($comp['QTSUBPRODROT'])))
                            );
                            
                        $product = $this->productFactory->getCodeProduct($itemComp['code']);
                        $itemComp['externalCode'] = $product['CDPRODINTE'];
                        $itemComp['unit'] = $product['SGUNIDADE'];
                        array_push($itensEst['itens'], json_decode(json_encode($itemComp), true));
                   }
                }
            }
            
            if((isset($i['IDUTILRETPRODACAB']) && $i['IDUTILRETPRODACAB'] == 'S') || empty($composicao)){
                
                $prodPai = array(
                        'name'           => $i['NMPRODUTO'],
                        'code'           => $i['CDPRODUTO'],
                        'externalCode'   => '',
                        'unit'           => '',
                        'quantity'       => floatval($this->util->tofloat($i['QTITVENDAEST']))
                    );
                    
                $product = $this->productFactory->getCodeProduct($prodPai['code']);
                $prodPai['externalCode'] = $product['CDPRODINTE'];
                $prodPai['unit'] = $product['SGUNIDADE'];
                array_push($itensEst['itens'], json_decode(json_encode($prodPai), true));
 
            }
            
        }

        return $itensEst;
    }
}